﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Problems.DataAnalysis.Symbolic.Regression;

namespace HeuristicLab.OSGAEvaluator {
  [Item("AdjustedEvaluatedSolutionsAnalyzer", "Analyzer which aggregates the adjusted evaluated solutions count from the predictive evaluator.")]
  [StorableType("791A5594-FC15-41AF-9D06-86D1313C4C8B")]
  public class AdjustedEvaluatedSolutionsAnalyzer : SymbolicDataAnalysisSingleObjectiveAnalyzer {
    private const string EvaluatorParameterName = "Evaluator";
    private const string AdjustedEvaluatedSolutionsResultName = "AdjustedEvaluatedSolutions";

    public ILookupParameter<SymbolicRegressionSingleObjectiveEvaluator> EvaluatorParameter {
      get { return (ILookupParameter<SymbolicRegressionSingleObjectiveEvaluator>)Parameters[EvaluatorParameterName]; }
    }

    public SymbolicRegressionSingleObjectiveEvaluator Evaluator {
      get { return EvaluatorParameter.ActualValue; }
    }

    public AdjustedEvaluatedSolutionsAnalyzer() {
      Parameters.Add(new LookupParameter<SymbolicRegressionSingleObjectiveEvaluator>(EvaluatorParameterName));
    }

    [StorableConstructor]
    protected AdjustedEvaluatedSolutionsAnalyzer(StorableConstructorFlag _) : base(_) { }

    protected AdjustedEvaluatedSolutionsAnalyzer(AdjustedEvaluatedSolutionsAnalyzer original, Cloner cloner) : base(original, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AdjustedEvaluatedSolutionsAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var evaluator = Evaluator as SymbolicRegressionSingleObjectiveOsgaEvaluator;

      if (evaluator == null)
        return base.Apply();

      var results = ResultCollectionParameter.ActualValue;
      if (!results.ContainsKey(AdjustedEvaluatedSolutionsResultName)) {
        results.Add(new Result(AdjustedEvaluatedSolutionsResultName, new DoubleValue(0)));
      }
      var adjustedEvaluatedSolutions = (DoubleValue)results[AdjustedEvaluatedSolutionsResultName].Value;
      adjustedEvaluatedSolutions.Value += evaluator.AdjustedEvaluatedSolutions;
      evaluator.AdjustedEvaluatedSolutions = 0;

      return base.Apply();
    }
  }
}
